// Binary heap functions
//
// R. Brette (Feb 2006)
// brette@di.ens.fr
//
//
// In all functions:
// H = heap (list)
// I = (sparse) vector of indexes in the heap
// (I(n)=m means element n is stored at location m in the heap
//
// Elements in the heap are vectors with
// 1st column = priority (time); 2nd column = index (id)
//
function [H,I]=emptyHeap(n)
// Returns an empty heap
//
// n = maximum number of elements

I=zeros(n,1); // spzeros?
H=list();
endfunction

function [H,I]=insertHeap(H,I,e)
// Inserts element e in the heap
//
// e = element

  H($+1)=e;
  i=size(H);
  while (i>1 & e(1)<H(floor(i/2))(1))
    H(i)=H(i/2);
    I(H(i/2)(2))=i;
    i=floor(i/2);
  end
  H(i)=e;
  I(e(2))=i;
endfunction

function [e,H,I]=popHeap(H,I)
// Returns element of highest priority (lowest value in the heap)

  e=H(1);
  I(e(2))=0;
  eLast=H($);
 
  j=1;
	i=2;
	n=size(H);
	
	if (n>1)
    	while (i<n)
    		if ((i<n-1) & (H(i+1)(1)<H(i)(1)))
    		  i=i+1;
    		end
    		if (eLast(1)<H(i)(1))
    		  break;
    		end
    		H(j)=H(i);
    		I(H(i)(2))=j;
    		j=i;
    		i=i*2;
    	end
    	H(j)=H(n);
    	I(H(n)(2))=j;
	end
	H(n)=null();
endfunction

function [H,I]=removeHeap(H,I,m)
// Removes element number m from the heap

  eLast=H($);
 
  j=I(m);
	i=j*2;
  I(m)=0;
	n=size(H);
	
	if (j<>n)
    	while (i<n)
    		if ((i<n-1) & (H(i+1)(1)<H(i)(1)))
    		  i=i+1;
    		end
    		if (eLast(1)<H(i)(1))
    		  break;
    		end
    		H(j)=H(i);
    		I(H(i)(2))=j;
    		j=i;
    		i=i*2;
    	end
    	H(j)=H(n);
    	I(H(n)(2))=j;
	end
	H(n)=null();
endfunction

function [H,I]=rescheduleHeap(H,I,n,t)
// Changes the priority of element number n into t
// (This is the naive way - very slow!)
  e=H(I(n));
  [H,I]=removeHeap(H,I,n);
  e(1)=t;
  [H,I]=insertHeap(H,I,e);
endfunction

