// Event-driven simulation of spiking neural networks
//
// R. Brette (brette@di.ens.fr)
// Feb 2006
//

exec('BinaryHeap.sci');

function spikes=EventDriven(X,W,T,fUpdate,fNextSpike,fOutgoingSpike,fIncomingSpike,dur)
// X = state matrix
// W = weight matrix (sparse)
//   W(i,j) = weight of connection j->i
// T = type of neurons (vector)
// fUpdate(X,t) = function for updating state
// fNextSpike(X) = gives time of the next spike
// fOutgoingSpike(X) = updates following an outgoing spike
// fIncomingSpike(X,w,t) = updates following an incoming spike
//  (t = type; w = weight)
// dur = duration

  // Initialization
  [n,m]=size(X);
  [H,I]=emptyHeap(n); // List of future events

  for i=1:n
    timing=fNextSpike(X(i,:));
    if timing>=0
      [H,I]=insertHeap(H,I,[timing,i]);
    end
  end
  spikes=[];
  t=0;
  
  // Let's go!
  while t<dur
    // Extract next event
    if size(H)==0
      printf("No more spike!\n");
      break;
    end
    [e,H,I]=popHeap(H,I);
    t=e(1);
    i=e(2);
    spikes=[spikes;i,t];
    
    // Update and spike
    X(i,:)=fUpdate(X(i,:),t);
    X(i,:)=fOutgoingSpike(X(i,:));
    
    // Recompute spike timing
    timing=fNextSpike(X(i,:));
    if (timing>=0)
      [H,I]=insertHeap(H,I,[timing,i]);
    end
    
    // Propagate spike
    target=find(W(:,i)<>0);
    // Update target
    X(target,:)=fIncomingSpike(fUpdate(X(target,:),t),W(target,i),T(i));
    for j=1:length(target)     
      // Recompute spike timing
      timing=fNextSpike(X(j,:));
      if (timing>=0)
        if (I(j)>0)
          [H,I]=rescheduleHeap(H,I,j,timing);
        else
          [H,I]=insertHeap(H,I,[timing,j]);
        end
      else
        if (I(j)>0)
          [H,I]=removeHeap(H,I,j);
        end
      end    
    end
  end

endfunction

