// Random network of IF with exponential synaptic conductances
//
// See "Exact simulation of integrate-and-fire models with
// exponential synaptic conductances", R. Brette, Neural Computation (2006).
//
// R. Brette (brette@di.ens.fr)
// Feb 2006
//
exec('ExpCondIF.sci');
exec('EventDriven.sci');

// Parameters
// (these can be included in the state variable X in case all neurons don't share
// the same values)
Vr = 0; // reset (= V0)
taus = 0.3; // relative to membrane time constant
Ep = 5; // excitatory synaptic reversal potential (relative to threshold)
Em = 0; // inhibitory synaptic reversal potential (=V0)
N=400; // 200 neurons
Ne=200; // 100 excitatory neurons
Ni=200; // 100 inhibitory neurons

X=[zeros(N,1),grand(N,1,'def')*6,grand(N,1,'def')*5,zeros(N,1)];
W=sprand(N,N,0.1,'uniform')*0.2; // 10% connection probability (sparse matrix)
W(:,(Ne+1):N)=-W(:,(Ne+1):N);   // inhibitory weights are negative
T=zeros(N,1);   // (irrelevant here)

// Please note: time is in units of the membrane time constant
// (e.g. 10 is say 200 ms)
timer();
spikes=EventDriven(X,W,T,fUpdate_ExpCondIF,fNextSpike_ExpCondIF,fOutgoingSpike_ExpCondIF,fIncomingSpike_ExpCondIF,10);
timer()

clf;
plot(spikes(:,2),spikes(:,1),'.');

