// Rho function
// Special function based the incomplete Gamma integral
// See "Exact simulation of integrate-and-fire models with
// exponential synaptic conductances", R. Brette, Neural Computation (2006).
//
// Here rho(g,taus) = \rho(1-\tau_s,\tau_s * g)
//
// R. Brette (brette@di.ens.fr)
// Feb 2006
//
function s=rho(g,taus)
// g can be a vector (not taus)
//
// A C function would be faster!
  MAX_GAMMA_ERROR=1e-15;
  
  x=taus*g;
  ap=1-taus;
  s=1/(1-taus) + 0*x;
  del=s;

  while (or(del >= s * MAX_GAMMA_ERROR))
		ap=ap+1;
     del=del .* x / ap;
     s=s+del;
  end
endfunction

